@@ -20,6 +20,7 @@ warnings.simplefilter('ignore', BiopythonWarning)
 
 from . import mapobject as mobj
 from . import pdbobject as pobj
+from . import mapfunctions as mfun
 
 class MapLoader(object):
     def __init__(self, pdb_code, directory="", cif=False):
@@ -35,6 +36,11 @@ class MapLoader(object):
         self.values_loaded = False
         self.values_loading = False
         self.has_both = True
+        self.mfunc = None
+        self.mean = 0
+        self.std = 1
+        self.mean_diff = 0
+        self.std_diff = 1
         # PRIVATE INTERFACE
         self._directory = directory        
         self._cif=cif
@@ -65,20 +71,26 @@ class MapLoader(object):
         self.load_pdb()
         if 'x-ray' in self.mobj.exp_method:
             if exists(self._filepath_ccp4) and exists(self._filepath_diff) and exists(self._filepath_ccp4+".done.txt"):
-                self.em_loaded = True
+                #self.em_loaded = True
                 return True        
             else:
                 return False
         elif 'electron' in self.mobj.exp_method.lower():
             if exists(self._filepath_ccp4) and exists(self._filepath_ccp4+".done.txt"):
-                self.em_loaded = True
+                #self.em_loaded = True
                 return True
             else:
                 return False
         else:
-            self.em_loaded = True
+            #self.em_loaded = True
             return True # it doesn;t NOT exists anyway
     
+    def success(self):
+        if exists(self._filepath) and exists(self._filepath_ccp4):
+            return True
+        else:
+            return False
+
     def download(self):                
         if not self.exists_pdb():
             if not self.already_started_pdb():
@@ -105,10 +117,18 @@ class MapLoader(object):
             os.remove(self._filepath+".start.txt")
         if exists(self._filepath_ccp4+".start.txt"):
             os.remove(self._filepath_ccp4+".start.txt")
-        with open(self._filepath + ".done.txt","w") as fw_pdb:
-            fw_pdb.write("done")
-        with open(self._filepath_ccp4 + ".done.txt","w") as fw_map:
-            fw_map.write("done")
+        if exists(self._filepath):
+            with open(self._filepath + ".done.txt","w") as fw_pdb:
+                fw_pdb.write("done")
+        else:
+            with open(self._filepath + ".failed.txt","w") as fw_pdb:
+                fw_pdb.write("failed")
+        if exists(self._filepath_ccp4):
+            with open(self._filepath_ccp4 + ".done.txt","w") as fw_map:
+                fw_map.write("done")
+        else:
+            with open(self._filepath_ccp4 + ".failed.txt","w") as fw_map:
+                fw_map.write("failed")
                 
     def download_pdb(self):
         self._fetch_pdbdata()
@@ -121,7 +141,7 @@ class MapLoader(object):
         elif 'electron' in self.mobj.exp_method:            
             print("Now downloading em matrix")
             self._fetch_maplink_em()
-        self.em_loaded = True
+        #self.em_loaded = True
     
     def load(self):
         self.pdb_loaded = self.load_pdb()
@@ -157,14 +177,16 @@ class MapLoader(object):
         # with the structure create a pdbobject
         return True
 
-    def load_map(self):
+    def load_map(self):        
         try:
             with open(self._filepath_ccp4, mode='rb') as file:
                 self._ccp4_binary = file.read()
             if self.has_both:
                 with open(self._filepath_diff, mode='rb') as file:
                     self._diff_binary = file.read()
-            self._create_mapheader()            
+            
+            self._create_mapheader(self._ccp4_binary)            
+            
             return True
         except Exception as e:
             print("Error loading map", str(e))
@@ -181,21 +203,26 @@ class MapLoader(object):
                     print("Waiting for load values", len(self.mobj.values), len(self.mobj.diff_values))
         return self.values_loaded
 
-    def load_values(self, diff=True):
+    def load_values(self, diff=True, adj_zero=True):
         try:            
             self.values_loading = True
-            self._create_mapvalues(False)
+            self._create_mapvalues(False,adj_zero=adj_zero)
             if diff and self.has_both:
-                self._create_mapvalues(True)            
+                self._create_mapvalues(True,adj_zero=adj_zero)
             if not self.has_both:
                 self.mobj.diff_values = []
             self.values_loading = False
             self.values_loaded = True
-        except:        
+        except Exception as e:
+            print(str(e))
             self.values_loaded = False
             self.values_loading = False
 
 
+    def get_or_make_func(self, interp):
+        if self.mfunc == None:
+            self.mfunc = mfun.MapFunctions(self.mobj.pdb_code,self.mobj,self.pobj, interp) #the default method is linear
+        return self.mfunc
     #################################################
     ############ PRIVATE INTERFACE ##################
     #################################################
@@ -207,13 +234,16 @@ class MapLoader(object):
             return False
         return True
         
-    def _fetch_maplink_xray(self):                
-        if not exists(self._filepath_ccp4):            
-            urllib.request.urlretrieve(self.mobj.ccp4_link, self._filepath_ccp4)
-        if not exists(self._filepath_diff):            
-            urllib.request.urlretrieve(self.mobj.diff_link, self._filepath_diff)
-        self.mobj.em_code = self.mobj.pdb_code
-        self.em_code = self.mobj.pdb_code
+    def _fetch_maplink_xray(self):
+        try:
+            if not exists(self._filepath_ccp4):            
+                urllib.request.urlretrieve(self.mobj.ccp4_link, self._filepath_ccp4)
+            if not exists(self._filepath_diff):            
+                urllib.request.urlretrieve(self.mobj.diff_link, self._filepath_diff)
+            self.mobj.em_code = self.mobj.pdb_code
+            self.em_code = self.mobj.pdb_code
+        except Exception as e:
+            print(e)
 
                         
     def _fetch_maplink_em(self):     
@@ -235,7 +265,7 @@ class MapLoader(object):
             print("getting",self._filepath_ccp4)
             if not exists(filepath_gz):
                 # need to download zipped file
-                print("getting",filepath_gz)
+                print("getting",self.mobj.ccp4_link,"to",filepath_gz)
                 try:
                     urllib.request.urlretrieve(self.mobj.ccp4_link, filepath_gz)
                 except Exception as e:
@@ -249,9 +279,9 @@ class MapLoader(object):
                 
         
 
-    def _create_mapheader(self):
+    def _create_mapheader(self, ccp4_binary):
         num_labels = 0
-        num_sym = 0
+        num_sym = 0        
         headers = [] #https://www.ccp4.ac.uk/html/maplib.html#description
         xheaders = [] 
         self.mobj.header_as_string = ""
@@ -296,13 +326,13 @@ class MapLoader(object):
             val = ""
             if not header == "X":
                 if typ == "int":
-                    val = int.from_bytes(self._ccp4_binary[i:i+inc], byteorder='little', signed=True)
+                    val = int.from_bytes(ccp4_binary[i:i+inc], byteorder='little', signed=True)
                     self.mobj.map_header[header] = val
                 elif typ == "double":
-                    val = struct.unpack('f', self._ccp4_binary[i:i+inc])[0]
+                    val = struct.unpack('f', ccp4_binary[i:i+inc])[0]
                     self.mobj.map_header[header] = val
                 elif typ == "string":
-                    val = self._ccp4_binary[i:i+inc].decode("utf-8") 
+                    val = ccp4_binary[i:i+inc].decode("utf-8") 
                     self.mobj.map_header[header] = val
                     
                 if len(header) > 7:
@@ -328,13 +358,13 @@ class MapLoader(object):
         for header, typ,inc  in xheaders:
             if not header == "X":
                 if typ == "int":
-                    val = int.from_bytes(self._ccp4_binary[i:i+inc], byteorder='little', signed=True)
+                    val = int.from_bytes(ccp4_binary[i:i+inc], byteorder='little', signed=True)
                     self.mobj.map_header[header] = val
                 elif typ == "double":
-                    val = struct.unpack('f', self._ccp4_binary[i:i+inc])[0]
+                    val = struct.unpack('f', ccp4_binary[i:i+inc])[0]
                     self.mobj.map_header[header] = val
                 elif typ == "string":
-                    val = self._ccp4_binary[i:i+inc].decode("utf-8") 
+                    val = ccp4_binary[i:i+inc].decode("utf-8") 
                     self.mobj.map_header[header] = val
                     
                 if len(header) > 7:
@@ -344,7 +374,7 @@ class MapLoader(object):
             i+=inc
         
                         
-    def _create_mapvalues(self, diff):
+    def _create_mapvalues(self, diff, adj_zero=True):
         use_binary = self._ccp4_binary
         vals = []
         #vals = {} #zeros alternative
@@ -360,6 +390,7 @@ class MapLoader(object):
 
         if diff:
             self.mobj.diff_values = np.zeros((self.mobj.F,self.mobj.M,self.mobj.S))
+            self.mobj.diff_has = self.mobj.F*self.mobj.M*self.mobj.S
         else:
             self.mobj.values = np.zeros((self.mobj.F,self.mobj.M,self.mobj.S))
 
@@ -374,7 +405,14 @@ class MapLoader(object):
                         self.mobj.diff_values[f,m,s] = val
                     else:
                         self.mobj.values[f,m,s] = val
-                    
+                            
+        #if adj_zero:
+        #    if diff:
+        #        zro = 
+        #        self.mobj.diff_values = np.zeros((self.mobj.F,self.mobj.M,self.mobj.S))
+        #    else:
+        #        self.mobj.values = np.zeros((self.mobj.F,self.mobj.M,self.mobj.S))
+
         """
         Blength = self.mobj.map_header["01_NC"] * self.mobj.map_header["02_NR"] * self.mobj.map_header["03_NS"]
         Bstart = len(self._ccp4_binary) - (4 * Blength)
